#!/bin/sh

set -ax

CK_DEFAULT_TIMEOUT="20"
GST_PLUGIN_LOADING_WHITELIST="gstreamer"
GST_REGISTRY=~/.cache/gstreamer-1.0/elements_queue.registry
GST_STATE_IGNORE_ELEMENTS=""
exec /usr/libexec/installed-tests/gstreamer/elements_queue
